
local harbour = ...

-- Create default building part registering function 
function registerDefaultBuildingPart(_nodePrefix)
	harbour:register({
		DataType = "BUILDING_PART",
		Id = _nodePrefix .. "_PART",
		Mover = { DataType = "BUILDING_PART_MOVER" },
		ConstructorData = {
                     DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
                     CoreObjectPrefab = _nodePrefix .. "_PREFAB"
                }
	})
end

function registerColor(_colorFbxName, _colorAssetId, _boatCompFlagNumber)

-- Register common prefab nodes
    harbour:registerAssetId("models/poleFlag" .. _colorFbxName .. ".fbx/Prefab/poleFlag/Pole/step_0_RootPart", HarbourPrefix .. "_FLAG" .. _colorAssetId .. "_ROOT_PREFAB")
    harbour:registerAssetId("models/poleFlag" .. _colorFbxName .. ".fbx/Prefab/poleFlag/Pole/step_3_TopPart", HarbourPrefix .. "_FLAG" .. _colorAssetId .. "_TOP_PREFAB")
    harbour:registerAssetId("models/poleFlag" .. _colorFbxName .. ".fbx/Prefab/poleFlag/Pole/step_2_Tiling2Part", HarbourPrefix .. "_FLAG" .. _colorAssetId .. "_TILING2_PREFAB")
    harbour:registerAssetId("models/poleFlag" .. _colorFbxName .. ".fbx/Prefab/poleFlag/Pole/step_1_Tiling3Part", HarbourPrefix .. "_FLAG" .. _colorAssetId .. "_TILING3_PREFAB")

    local defaultNodePrefixList = {
	HarbourPrefix .. "_FLAG" .. _colorAssetId .. "_TOP",
	HarbourPrefix .. "_FLAG" .. _colorAssetId .. "_TILING2",
	HarbourPrefix .. "_FLAG" .. _colorAssetId .. "_TILING3"
    }

-- Register simple building part assets
    for i, nodePrefix in ipairs(defaultNodePrefixList) do
	registerDefaultBuildingPart(nodePrefix)
    end

-- Register the pole flag part
    harbour:register({
	DataType = "BUILDING_PART",
	Id = HarbourPrefix .. "_FLAG" .. _colorAssetId .. "_ROOT_PART",
	Name = HarbourPrefix .. "_FLAG" .. _colorAssetId .. "_ROOT_NAME",
	Description = HarbourPrefix .. "_FLAG" .. _colorAssetId .. "_ROOT_DESC",
	Category = "CORE",
	Mover = { DataType = "BUILDING_PART_MOVER_INSTANCE" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_SCALER", -- Since there are different types of building constructor, 'DataType' is necessary here
		CoreObjectPrefab = HarbourPrefix .. "_FLAG" .. _colorAssetId .. "_ROOT_PREFAB",
		EndPart = HarbourPrefix .. "_FLAG" .. _colorAssetId .. "_TOP_PART",
		FillerList = {
			HarbourPrefix .. "_FLAG" .. _colorAssetId .. "_TILING2_PART"
                },
		BasementFillerList = {
			HarbourPrefix .. "_FLAG" .. _colorAssetId .. "_TILING3_PART"
         	},
		MinimumScale = 0,
		BasementScale = 1,
		IsVertical = true
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
                        { Resource = "WOOD", Quantity = 2 },
			{ Resource = "WOOLEN_CLOTH", Quantity = 2 }
		}
	},
        DesirabilityModifiers = {
                {
                        Desirability = "DESIRABILITY_RESIDENTIAL",
                        Radius = 20,
                        Value = 0.1
                }
        }
    })

-- Override building part components properties (generated by the building asset processor) to add building zones

    harbour:registerPrefabComponent("models/poleFlag" .. _colorFbxName .. ".fbx/Prefab/poleFlag/Pole/step_0_RootPart", {
	DataType = "COMP_BUILDING_PART",
	HasBuildingZone = true,
	BuildingZone = { 1, 1 }
    })

    --------------------------------------------------------------------------
    -- Register waterline component
    harbour:registerPrefabComponent("models/poleFlag" .. _colorFbxName .. ".fbx/Prefab/poleFlag/Pole/step_0_RootPart", {
	DataType = HarbourPrefix .. "_COMP_FLAG" .. _boatCompFlagNumber,
	Enabled = true
    })

    harbour:registerPrefabComponent("models/poleFlag" .. _colorFbxName .. ".fbx/Prefab/poleFlag/Pole/step_0_RootPart", {
	DataType = "COMP_GROUNDED",
	GroundToWater = true
    })

end

registerColor("Red", "_RED", 1)
registerColor("Blue", "_BLUE", 2)
--registerColor("Green", "_GREEN", 3)
--registerColor("Purple", "_PURPLE", 4)
--registerColor("Orange", "_ORANGE", 5)
--registerColor("Turquoise", "_TURQUOISE", 6)
--------------------------------------------------------------------------

--------------------------------------------------------------------------

